package chess;

import java.util.HashMap;
import java.util.Map;

import chess.board.Board;
import chess.board.Move;
import chess.board.PlayerColor;
import chess.graphics.GuiStartScreen;
import chess.graphics.PlayerType;
import chess.player.AiPlayer;
import chess.player.GuiPlayer;
import chess.player.NetworkPlayer;
import chess.player.Player;

/**Chess Server that runs the game
 * @author Garon
 *
 */
public class ChessServer {

	/**Main Game loop
	 * @param args
	 */
	public static void main(String[] args) {
		while(true){
			//Creates a board and map of players
			Board test = new Board();
			Map<PlayerColor, Player> playerMap = new HashMap<PlayerColor, Player>();
			
			GuiStartScreen g = new GuiStartScreen("Chess Start Screen");
			g.setVisible(true);
					
			//Doesn't start game till setting are set
			while(!g.isSet){
				try {
					Thread.sleep(50);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}			
			}	
			
			if(g.whitePanel.player == PlayerType.GUI){
				playerMap.put(PlayerColor.WHITE, new GuiPlayer(PlayerColor.WHITE));
			}
			else if(g.whitePanel.player == PlayerType.NET){
				playerMap.put(PlayerColor.WHITE, new NetworkPlayer(PlayerColor.WHITE));
			}
			else if(g.whitePanel.player == PlayerType.AI){			
				playerMap.put(PlayerColor.WHITE, new AiPlayer(PlayerColor.WHITE,g.whiteDiff,g.whiteStyle));
			}
			
			if(g.blackPanel.player == PlayerType.GUI){
				playerMap.put(PlayerColor.BLACK, new GuiPlayer(PlayerColor.BLACK));
			}
			else if(g.blackPanel.player == PlayerType.NET){
				playerMap.put(PlayerColor.BLACK, new NetworkPlayer(PlayerColor.BLACK));
			}
			else if(g.blackPanel.player == PlayerType.AI){
				playerMap.put(PlayerColor.BLACK, new AiPlayer(PlayerColor.BLACK,g.blackDiff,g.blackStyle));
			}		
			
			//Starts the game for both players
			playerMap.get(PlayerColor.WHITE).startGame(PlayerColor.WHITE);
			playerMap.get(PlayerColor.BLACK).startGame(PlayerColor.BLACK);
			
			boolean gameTerminated = false;	//Boolean to determine if game still in progress
			
			PlayerColor currentColor = PlayerColor.WHITE;	//Player whose turn it is
			
			//Main game loop, continues to run till someone placed in checkmate
			while(!gameTerminated){
				Player currentPlayer = playerMap.get(currentColor);		
				boolean moveExecuted = false;
				Move currentMove = null;
				while(!moveExecuted){	//Will ask for a valid move, till one has been made
					try{
					currentMove = currentPlayer.getMove();				
					moveExecuted = test.movePiece(currentColor, currentMove.startPoint, currentMove.endPoint);
					}catch(Exception e){
						System.out.println(e);
						moveExecuted = false;
					}
				}			
				currentColor = Board.switchColor(currentColor);	
				Player nextPlayer = playerMap.get(currentColor);	//makes it other players turn
				nextPlayer.sendMove(currentMove);					//sends them the move that was just made
				if(test.isCheckmate(currentColor)){					//tests if they are in checkmate
					gameTerminated = true;
					playerMap.get(currentColor).gameTerminated(false);
					playerMap.get(Board.switchColor(currentColor)).gameTerminated(true);
					
				}
			}			
			
			
		}
	}	

}
