package chess.board;

import java.awt.Point;
import java.util.ArrayList;

import chess.board.pieces.Bishop;
import chess.board.pieces.King;
import chess.board.pieces.Knight;
import chess.board.pieces.Pawn;
import chess.board.pieces.Piece;
import chess.board.pieces.Queen;
import chess.board.pieces.Rook;

/**Board Class,  holds board information
 * @author Garon
 *
 */
public class Board {

	private Piece[][] board;	//Array holding the pieces

	/**Constructor for board
	 * 
	 */
	public Board() {
		
		board = new Piece[8][8];	//initializes arrays

		//Sets all the pieces into place on the board
		board[0][0] = new Rook(PlayerColor.WHITE);
		board[7][0] = new Rook(PlayerColor.WHITE);
		board[7][7] = new Rook(PlayerColor.BLACK);
		board[0][7] = new Rook(PlayerColor.BLACK);

		board[1][0] = new Knight(PlayerColor.WHITE);
		board[6][0] = new Knight(PlayerColor.WHITE);
		board[1][7] = new Knight(PlayerColor.BLACK);
		board[6][7] = new Knight(PlayerColor.BLACK);

		board[2][0] = new Bishop(PlayerColor.WHITE);
		board[5][0] = new Bishop(PlayerColor.WHITE);
		board[2][7] = new Bishop(PlayerColor.BLACK);
		board[5][7] = new Bishop(PlayerColor.BLACK);

		board[3][0] = new Queen(PlayerColor.WHITE);
		board[3][7] = new Queen(PlayerColor.BLACK);
		board[4][0] = new King(PlayerColor.WHITE);
		board[4][7] = new King(PlayerColor.BLACK);

		board[0][1] = new Pawn(PlayerColor.WHITE);
		board[1][1] = new Pawn(PlayerColor.WHITE);
		board[2][1] = new Pawn(PlayerColor.WHITE);
		board[3][1] = new Pawn(PlayerColor.WHITE);
		board[4][1] = new Pawn(PlayerColor.WHITE);
		board[5][1] = new Pawn(PlayerColor.WHITE);
		board[6][1] = new Pawn(PlayerColor.WHITE);
		board[7][1] = new Pawn(PlayerColor.WHITE);

		board[0][6] = new Pawn(PlayerColor.BLACK);
		board[1][6] = new Pawn(PlayerColor.BLACK);
		board[2][6] = new Pawn(PlayerColor.BLACK);
		board[3][6] = new Pawn(PlayerColor.BLACK);
		board[4][6] = new Pawn(PlayerColor.BLACK);
		board[5][6] = new Pawn(PlayerColor.BLACK);
		board[6][6] = new Pawn(PlayerColor.BLACK);
		board[7][6] = new Pawn(PlayerColor.BLACK);


	}

	/**Clone Constructor
	 * @param b	Board to be cloned
	 */
	public Board(Board b) {
		board = new Piece[8][8];
		for (int i = 0; i < board.length; i++) {
			for (int j = 0; j < board.length; j++) {
				try {
					if(b.board[i][j] != null){
					board[i][j] = (Piece) b.board[i][j].clone();
					}
				} catch (CloneNotSupportedException e) {
					e.printStackTrace();
				}
			}
		}
	}


	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {

		StringBuilder s = new StringBuilder();

		for (int i = board.length - 1; i >= 0; i--) {
			for (int j = 0; j < board[0].length; j++) {

				Piece temp = board[j][i];
				if (temp == null) {
					s.append("[ ]");
				} else {
					s.append('[' + temp.toString() + ']');
				}
			}
			s.append('\n');
		}
		return s.toString();
	}
	
	/**Converts a pixel point into an index location on the piece array
	 * @param P		Point to be converted
	 * @return		Converted Point
	 */
	public Point convertToIndex(Point P){
		Point indexP =  new Point();
		indexP.x = (P.x)/50 - 1;
		indexP.y = 8 - (int)((P.y)/50);
		return indexP;
	}
	
	/**Moves a piece on the board
	 * @param c		Player trying to move piece
	 * @param p1	Position of piece to be moved
	 * @param p2	Position to be moved to
	 * @return		True if can be moved, false if it cannot
	 */
	public boolean movePiece(PlayerColor c, Point p1, Point p2){
		if(board[p1.x][p1.y] != null && board[p1.x][p1.y].getOwner() == c && 
		   board[p1.x][p1.y].isMoveValid(p1, p2, this)){
			Board scratch = new Board(this);
			scratch.getBoard()[p2.x][p2.y] = scratch.getBoard()[p1.x][p1.y];
			scratch.getBoard()[p1.x][p1.y] = null;
			if(scratch.isCheck(scratch.getBoard()[p2.x][p2.y].owner)){
				return false;
			}				
			board[p2.x][p2.y] = board[p1.x][p1.y];	//if piece can be moved, it moves it
			board[p1.x][p1.y] = null;
			return true;
		}
		else{
			return false;
		}		
	}
	
	/**Gets all the Possible moves from the current board state
	 * @param c		Player who's turn it is
	 * @return		Array of all possible moves
	 */
	public Move[] getAllPossibleMoves(PlayerColor c){
		ArrayList<Move> possibleMoves = new ArrayList<Move>(); 
		for(int i = 0; i < 8; i++){
			for(int j = 0; j < 8; j++){
				if(board[j][i] != null && board[j][i].getOwner() == c){
					Move[] pieceMoves = board[j][i].getPossibleMoves(new Point(j,i), this);
					for(int k = 0; k < pieceMoves.length; k++){
						possibleMoves.add(pieceMoves[k]);
					}
				}
			}
		}
		return (Move[]) possibleMoves.toArray(new Move[0]);		
	}	
	/**Gets all the Possible moves from the current board state
	 * @param c		Player who's turn it is
	 * @return		Array of all possible moves
	 */
	public Move[] getAllLegalMoves(PlayerColor c){
		ArrayList<Move> possibleMoves = new ArrayList<Move>();

		for(int i = 0; i < 8; i++){
			for(int j = 0; j < 8; j++){
				if(board[j][i] != null && board[j][i].getOwner() == c){
					Move[] pieceMoves = board[j][i].getPossibleMoves(new Point(j,i), this);
					for(int k = 0; k < pieceMoves.length; k++){
						Board scratch = new Board(this);
						Move move = pieceMoves[k];
						scratch.getBoard()[move.endPoint.x][move.endPoint.y] = scratch.getBoard()[move.startPoint.x][move.startPoint.y];
						scratch.getBoard()[move.startPoint.x][move.startPoint.y] = null;
						if(!scratch.isCheck(c)){
							possibleMoves.add(move);
						}						
					}
				}
			}
		}

		return (Move[]) possibleMoves.toArray(new Move[0]);		
	}	
	
	/**Asks if a player is in check
	 * @param c	Player to be determined if in check or not
	 * @return	True if in check, false if not
	 */
	public boolean isCheck(PlayerColor c){
		Move[] allMoves = getAllPossibleMoves(switchColor(c));
		for(int i = 0; i < allMoves.length; i++){
			if(this.board[allMoves[i].endPoint.x][allMoves[i].endPoint.y] != null &&
			   ((c == PlayerColor.BLACK && 
			    this.board[allMoves[i].endPoint.x][allMoves[i].endPoint.y] instanceof King &&
			    this.board[allMoves[i].endPoint.x][allMoves[i].endPoint.y].owner == PlayerColor.BLACK) ||
			   (c == PlayerColor.WHITE &&
			    this.board[allMoves[i].endPoint.x][allMoves[i].endPoint.y] instanceof King &&
			    this.board[allMoves[i].endPoint.x][allMoves[i].endPoint.y].owner == PlayerColor.WHITE))){
				return true;
			}
		}		
		return false;
	}
	
	/**Asks if a player is in checkmate
	 * @param c	Player to be determined if in checkmate
	 * @return	True if in checkmate, false if not
	 */
	public boolean isCheckmate(PlayerColor c){
		Move[] allMoves = getAllPossibleMoves(c);
		Boolean isMate = false;
		for(int i = 0; i < allMoves.length; i++){
			Board scratch = new Board(this);
			scratch.movePiece(c, allMoves[i].startPoint, allMoves[i].endPoint);
			isMate = scratch.isCheck(c);
			if(isMate == false){
				return false;
			}
		}
		return true;
	}
	
	/**Switches the current Color
	 * @param c		Color asked to be switched
	 * @return		Black if c is white, White if c is black
	 */
	public static PlayerColor switchColor(PlayerColor c){
		if(c == PlayerColor.WHITE){
			return PlayerColor.BLACK;
		}
		else{
			return PlayerColor.WHITE;
		}		
	}

	/**Gets the board
	 * @return Returns the board.
	 */
	public Piece[][] getBoard() {
		return board;
	}

	/**Sets the board
	 * @param board The board to set.
	 */
	public void setBoard(Piece[][] board) {
		this.board = board;
	}


	
	

}
