package chess.board.pieces;

import java.awt.Image;
import java.awt.Point;
import java.util.ArrayList;

import javax.swing.ImageIcon;

import chess.board.Board;
import chess.board.Move;
import chess.board.PlayerColor;

/**Bishop Class, extends Piece
 * @author Garon
 *
 */
public class Bishop extends Piece {

	/**Constructor for Bishop
	 * @param owner	Owner of the Piece
	 */
	public Bishop(PlayerColor owner) {
		super(owner);
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#getPossibleMoves(chess.board.Board)
	 */
	@Override
	public Move[] getPossibleMoves(Point p1, Board b) {
		ArrayList<Move> possibleMoves = new ArrayList<Move>();
		int diagCounter = 1;
		while(p1.x + diagCounter != 8 && p1.y + diagCounter != 8 &&
			  b.getBoard()[p1.x + diagCounter][p1.y + diagCounter] == null){
			possibleMoves.add(new Move(p1, new Point(p1.x + diagCounter, p1.y + diagCounter)));
			diagCounter++;
		}
		if(p1.x + diagCounter != 8 && p1.y + diagCounter != 8 &&
			  b.getBoard()[p1.x + diagCounter][p1.y + diagCounter].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(p1.x + diagCounter, p1.y + diagCounter)));
		}
		diagCounter = 1;
		while(p1.x - diagCounter != -1 && p1.y - diagCounter != -1 && 
			  b.getBoard()[p1.x - diagCounter][p1.y - diagCounter] == null){
			possibleMoves.add(new Move(p1, new Point(p1.x - diagCounter, p1.y - diagCounter)));
			diagCounter++;
		}
		if(p1.x - diagCounter != -1 && p1.y - diagCounter != -1 &&
		   b.getBoard()[p1.x - diagCounter][p1.y - diagCounter].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(p1.x - diagCounter, p1.y - diagCounter)));
		}
		diagCounter = 1;
		while(p1.x + diagCounter != 8 && p1.y - diagCounter != -1 && 
			  b.getBoard()[p1.x + diagCounter][p1.y - diagCounter] == null){
			possibleMoves.add(new Move(p1, new Point(p1.x + diagCounter, p1.y - diagCounter)));
			diagCounter++;
			}
		if(p1.x + diagCounter != 8 && p1.y - diagCounter != -1 &&
		   b.getBoard()[p1.x + diagCounter][p1.y - diagCounter].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(p1.x + diagCounter, p1.y - diagCounter)));
		}
		diagCounter = 1;
		while(p1.x - diagCounter != -1 && p1.y + diagCounter != 8 && 
			  b.getBoard()[p1.x - diagCounter][p1.y + diagCounter] == null){
			possibleMoves.add(new Move(p1, new Point(p1.x - diagCounter, p1.y + diagCounter)));
			diagCounter++;
			}
		if(p1.x - diagCounter != -1 && p1.y + diagCounter != 8 &&
		   b.getBoard()[p1.x - diagCounter][p1.y + diagCounter].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(p1.x - diagCounter, p1.y + diagCounter)));
		}
		return (Move[]) possibleMoves.toArray(new Move[0]);
	}
	
	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#isMoveValid(java.awt.Point, java.awt.Point, chess.board.pieces.Piece[][])
	 */
	@Override
	public boolean isMoveValid(Point p1, Point p2, Board b) {
		boolean pieceInWay = false;	//used to check if there is a piece in the Bishop's path
		//Checks if the place the piece is moving is unocupied or occupied by an opponents piece
		if( ((b.getBoard()[p2.x][p2.y] == null) || (b.getBoard()[p2.x][p2.y].owner != this.owner)) &&
			p1.x - p2.x == p1.y - p2.y){
			int moveDistance = p1.x - p2.x;		//diagonal distance the bishop must move
			if(moveDistance < 0){				//checks the Bishop's SW path
				int moveCounter = -1;
				while(moveCounter != moveDistance){
					if(b.getBoard()[p1.x-moveCounter][p1.y-moveCounter] != null){
						pieceInWay = true;						
					}
					moveCounter--;
				}
			}
			else{								//checks the bishop's NE path
				int moveCounter = 1;
				while(moveCounter != moveDistance){
					if(b.getBoard()[p1.x-moveCounter][p1.y-moveCounter] != null){
						pieceInWay = true;
					}
					moveCounter++;
				}				
			}			
			return !pieceInWay;					//returns true if the path is clear
		}
		else if(((b.getBoard()[p2.x][p2.y] == null) || (b.getBoard()[p2.x][p2.y].owner != this.owner)) &&
			p1.x - p2.x == -(p1.y - p2.y)){
			int xMoveDistance = p1.x - p2.x;	//Diagonal distance the bishop must move	
			if(xMoveDistance < 0){				//checks the bishop's NW path
				int moveCounter = -1;
				while(moveCounter != xMoveDistance){
					if(b.getBoard()[p1.x-moveCounter][p1.y+moveCounter] != null){
						pieceInWay = true;
					}
					moveCounter--;
				}
			}
			else{								//checks the bishop's SE path
				int moveCounter = 1;				
				while(moveCounter != xMoveDistance){
					if(b.getBoard()[p1.x-moveCounter][p1.y+moveCounter] != null){
						pieceInWay = true;
					}					
					moveCounter++;
				}				
			}
			return !pieceInWay;					//returns true if the path is clear
		}
		else {
			return false;
		}
		
	}	
	
	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#getImage()
	 */
	@Override
	public Image getImage() {
		Image i = null;
		
		if(owner == PlayerColor.WHITE){
		i = new ImageIcon("chess/img/whiteBishop.png").getImage();
		}
		else {
		i = new ImageIcon("chess/img/blackBishop.png").getImage();
		}
		
		return i;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		if(owner == PlayerColor.WHITE)
			return "b";
		else {
			return "B";
		}
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		return new Bishop(owner);
	}



}
