package chess.board.pieces;

import java.awt.Image;
import java.awt.Point;
import java.util.ArrayList;

import javax.swing.ImageIcon;

import chess.board.Board;
import chess.board.Move;
import chess.board.PlayerColor;

/**King Class, extends Piece
 * @author Garon
 *
 */
public class King extends Piece {

	/**Constructor for King
	 * @param owner	Owner of the Piece
	 */
	public King(PlayerColor owner) {
		super(owner);		
	}

	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#getPossibleMoves(chess.board.Board)
	 */
	@Override
	public Move[] getPossibleMoves(Point p1, Board b) {
		ArrayList<Move> possibleMoves = new ArrayList<Move>();
		if(p1.x != 0){
			if(b.getBoard()[p1.x-1][p1.y] == null ||
			   b.getBoard()[p1.x-1][p1.y].owner != this.owner){
				   possibleMoves.add(new Move(p1, new Point(p1.x-1, p1.y)));
			   }
			if(p1.y != 0 && (b.getBoard()[p1.x-1][p1.y-1] == null ||
			   b.getBoard()[p1.x-1][p1.y-1].owner != this.owner)){
				possibleMoves.add(new Move(p1, new Point(p1.x-1, p1.y-1)));
			}				
			if(p1.y != 7 && (b.getBoard()[p1.x-1][p1.y+1] == null ||
			   b.getBoard()[p1.x-1][p1.y+1].owner != this.owner)){
				possibleMoves.add(new Move(p1, new Point(p1.x-1, p1.y+1)));
			}		
		}
		if(p1.x != 7){
			if(b.getBoard()[p1.x+1][p1.y] == null ||
			   b.getBoard()[p1.x+1][p1.y].owner != this.owner){
				   possibleMoves.add(new Move(p1, new Point(p1.x+1, p1.y)));
			   }
			if(p1.y != 0 && (b.getBoard()[p1.x+1][p1.y-1] == null ||
			   b.getBoard()[p1.x+1][p1.y-1].owner != this.owner)){
				possibleMoves.add(new Move(p1, new Point(p1.x+1, p1.y-1)));
			}				
			if(p1.y != 7 && (b.getBoard()[p1.x+1][p1.y+1] == null ||
			   b.getBoard()[p1.x+1][p1.y+1].owner != this.owner)){
				possibleMoves.add(new Move(p1, new Point(p1.x+1, p1.y+1)));
			}
		}
		if(p1.y != 0 &&(b.getBoard()[p1.x][p1.y-1] == null ||
		   b.getBoard()[p1.x][p1.y-1].owner != this.owner)){
			possibleMoves.add(new Move(p1, new Point(p1.x, p1.y-1)));			
		}
		if(p1.y != 7 &&(b.getBoard()[p1.x][p1.y+1] == null ||
		   b.getBoard()[p1.x][p1.y+1].owner != this.owner)){
			possibleMoves.add(new Move(p1, new Point(p1.x, p1.y+1)));			
		}
		return (Move[]) possibleMoves.toArray(new Move[0]);
	}
	
	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#isMoveValid(java.awt.Point, java.awt.Point, chess.board.pieces.Piece[][])
	 */
	@Override
	public boolean isMoveValid(Point p1, Point p2, Board b) {
		//Checks to see if move is a valid move for the king
		if(((b.getBoard()[p2.x][p2.y] == null) || (b.getBoard()[p2.x][p2.y].owner != this.owner)) &&
		   ((p2.x == p1.x && p2.y == p1.y + 1) ||
		    (p2.x == p1.x + 1 && p2.y == p1.y) ||
		    (p2.x == p1.x && p2.y == p1.y - 1) ||
		    (p2.x == p1.x - 1 && p2.y == p1.y) ||
		    (p2.x == p1.x + 1 && p2.y == p1.y + 1) ||
		    (p2.x == p1.x - 1 && p2.y == p1.y - 1) ||
		    (p2.x == p1.x + 1 && p2.y == p1.y - 1) ||
		    (p2.x == p1.x - 1 && p2.y == p1.y + 1))){
			return true;			
		}
		else {
			return false;
		}
	}	
	
	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#getImage()
	 */
	@Override
	public Image getImage() {
		Image i = null;
		
		if(owner == PlayerColor.WHITE){
		i = new ImageIcon("chess/img/whiteKing.png").getImage();
		}
		else {
		i = new ImageIcon("chess/img/blackKing.png").getImage();
		}
		
		return i;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		if(owner == PlayerColor.WHITE)
			return "k";
		else {
			return "K";
		}
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		return new King(owner);
	}



}
