package chess.board.pieces;

import java.awt.Image;
import java.awt.Point;
import java.util.ArrayList;

import javax.swing.ImageIcon;

import chess.board.Board;
import chess.board.Move;
import chess.board.PlayerColor;

/**Knight Class, extends Piece
 * @author Garon
 *
 */
public class Knight extends Piece {

	/**Constructor for Knight
	 * @param owner	Owner of the Piece
	 */
	public Knight(PlayerColor owner) {
		super(owner);
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#getPossibleMoves(chess.board.Board)
	 */
	@Override
	public Move[] getPossibleMoves(Point p1, Board b) {
		ArrayList<Move> possibleMoves = new ArrayList<Move>();
		if(p1.x >= 2){
			if(p1.y >= 1 && (b.getBoard()[p1.x-2][p1.y-1] == null ||
			   b.getBoard()[p1.x-2][p1.y-1].owner != this.owner)){
				possibleMoves.add(new Move(p1, new Point(p1.x-2, p1.y-1)));
			}
			if(p1.y <= 6 && (b.getBoard()[p1.x-2][p1.y+1] == null ||
			   b.getBoard()[p1.x-2][p1.y+1].owner != this.owner)){
				possibleMoves.add(new Move(p1, new Point(p1.x-2, p1.y+1)));
			}
		}
		if(p1.x <= 5){
			if(p1.y >= 1 && (b.getBoard()[p1.x+2][p1.y-1] == null ||
			   b.getBoard()[p1.x+2][p1.y-1].owner != this.owner)){
				possibleMoves.add(new Move(p1, new Point(p1.x+2, p1.y-1)));
			}
			if(p1.y <= 6 && (b.getBoard()[p1.x+2][p1.y+1] == null ||
			   b.getBoard()[p1.x+2][p1.y+1].owner != this.owner)){
				possibleMoves.add(new Move(p1, new Point(p1.x+2, p1.y+1)));
			}
		}
		if(p1.x >= 1){
			if(p1.y >= 2 && (b.getBoard()[p1.x-1][p1.y-2] == null ||
			   b.getBoard()[p1.x-1][p1.y-2].owner != this.owner)){
				possibleMoves.add(new Move(p1, new Point(p1.x-1, p1.y-2)));
			}
			if(p1.y <= 5 && (b.getBoard()[p1.x-1][p1.y+2] == null ||
			   b.getBoard()[p1.x-1][p1.y+2].owner != this.owner)){
				possibleMoves.add(new Move(p1, new Point(p1.x-1, p1.y+2)));
			}
		}
		if(p1.x <= 6){
			if(p1.y >= 2 && (b.getBoard()[p1.x+1][p1.y-2] == null ||
			   b.getBoard()[p1.x+1][p1.y-2].owner != this.owner)){
				possibleMoves.add(new Move(p1, new Point(p1.x+1, p1.y-2)));
			}
			if(p1.y <= 5 && (b.getBoard()[p1.x+1][p1.y+2] == null ||
			   b.getBoard()[p1.x+1][p1.y+2].owner != this.owner)){
				possibleMoves.add(new Move(p1, new Point(p1.x+1, p1.y+2)));
			}
		}
		return (Move[]) possibleMoves.toArray(new Move[0]);
	}
	
	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#isMoveValid(java.awt.Point, java.awt.Point, chess.board.pieces.Piece[][])
	 */
	@Override
	public boolean isMoveValid(Point p1, Point p2, Board b) {
		//Checks to see if move is one of the knight's eight valid moves
		if((b.getBoard()[p2.x][p2.y] == null || b.getBoard()[p2.x][p2.y].owner != this.owner) &&
		   ((p2.x == p1.x + 2 && p2.y == p1.y + 1) ||
			(p2.x == p1.x + 2 && p2.y == p1.y - 1) ||
			(p2.x == p1.x + 1 && p2.y == p1.y + 2) ||
			(p2.x == p1.x + 1 && p2.y == p1.y - 2) ||
			(p2.x == p1.x - 2 && p2.y == p1.y + 1) ||
			(p2.x == p1.x - 2 && p2.y == p1.y - 1) ||
			(p2.x == p1.x - 1 && p2.y == p1.y + 2) ||
			(p2.x == p1.x - 1 && p2.y == p1.y - 2))){
			return true;
		}
		else {
			return false;
		}		
	}
	
	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#getImage()
	 */
	@Override
	public Image getImage() {
		Image i = null;
		
		if(owner == PlayerColor.WHITE){
		i = new ImageIcon("chess/img/whiteKnight.png").getImage();
		}
		else {
		i = new ImageIcon("chess/img/blackKnight.png").getImage();
		}
		
		return i;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		if(owner == PlayerColor.WHITE)
			return "n";
		else {
			return "N";
		}
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		return new Knight(owner);
	}



}
