package chess.board.pieces;

import java.awt.Image;
import java.awt.Point;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import chess.board.Board;
import chess.board.Move;
import chess.board.PlayerColor;

/**Pawn Class, extends Piece
 * @author Garon
 *
 */
public class Pawn extends Piece {
	
	/**Constructor for Pawn
	 * @param owner	Owner of the Piece
	 */
	public Pawn(PlayerColor owner) {
		super(owner);		
	}

	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#getPossibleMoves(chess.board.Board)
	 */
	@Override
	public Move[] getPossibleMoves(Point p1, Board b) {
		ArrayList<Move> possibleMoves = new ArrayList<Move>();
		if(this.owner == PlayerColor.BLACK){			
			if(b.getBoard()[p1.x][p1.y-1] == null){
				possibleMoves.add(new Move(p1, new Point(p1.x, p1.y-1)));
				if(p1.y == 6){
					if(b.getBoard()[p1.x][p1.y-2] == null){
					possibleMoves.add(new Move(p1, new Point(p1.x, p1.y-2)));
					}
				}
			}
			if(p1.x != 0 &&
			   b.getBoard()[p1.x-1][p1.y-1] != null &&
			   b.getBoard()[p1.x-1][p1.y-1].owner == PlayerColor.WHITE){
				possibleMoves.add(new Move(p1, new Point(p1.x-1, p1.y-1)));
			}
			if(p1.x != 7 &&
			   b.getBoard()[p1.x+1][p1.y-1] != null && 
			   b.getBoard()[p1.x+1][p1.y-1].owner == PlayerColor.WHITE){
				possibleMoves.add(new Move(p1, new Point(p1.x+1, p1.y-1)));
			}
		}
		else{
			if(b.getBoard()[p1.x][p1.y+1] == null){
				possibleMoves.add(new Move(p1, new Point(p1.x, p1.y+1)));
				if(p1.y == 1){
					if(b.getBoard()[p1.x][p1.y+2] == null){
					possibleMoves.add(new Move(p1, new Point(p1.x, p1.y+2)));
					}
				}
			}
			if(p1.x != 0 &&
			   b.getBoard()[p1.x-1][p1.y+1] != null && 
			   b.getBoard()[p1.x-1][p1.y+1].owner == PlayerColor.BLACK){
				possibleMoves.add(new Move(p1, new Point(p1.x-1, p1.y+1)));
			}
			if(p1.x != 7 &&
			   b.getBoard()[p1.x+1][p1.y+1] != null && 
			   b.getBoard()[p1.x+1][p1.y+1].owner == PlayerColor.BLACK){
				possibleMoves.add(new Move(p1, new Point(p1.x+1, p1.y+1)));
			}
		}		
		return (Move[]) possibleMoves.toArray(new Move[0]);
	}
	
	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#isMoveValid(java.awt.Point, java.awt.Point, chess.board.pieces.Piece[][])
	 */
	@Override
	public boolean isMoveValid(Point p1, Point p2, Board b) {
		//Checks for black pawn moving for the first time
		if(p1.y == 6 && this.owner == PlayerColor.BLACK && 
	       b.getBoard()[p2.x][p2.y] == null && p2.x == p1.x && (p2.y == 4 || p2.y == 5)){
			return true;	
		}
		//Checks for white pawn moving for the first time
		else if(p1.y == 1 && this.owner == PlayerColor.WHITE && 
			    b.getBoard()[p2.x][p2.y] == null && p2.x == p1.x && (p2.y == 2 || p2.y == 3)){
			return true;
		}		
		//Checks black pawn movement
		else if(this.owner == PlayerColor.BLACK && b.getBoard()[p2.x][p2.y] == null && p2.y == p1.y - 1
				&& p2.x == p1.x){				
			if(p2.y == 0){		//Promotes the pawn to a queen if it reaches the other side
				b.getBoard()[p1.x][p1.y] = new Queen(PlayerColor.BLACK);
			}
			return true;	
		}
		//Checks white pawn movement
		else if(this.owner == PlayerColor.WHITE && b.getBoard()[p2.x][p2.y] == null && p2.y == p1.y + 1
				&& p2.x == p1.x){
			if(p2.y == 7){		//Promotes the pawn to a queen if it reaches the other side
				b.getBoard()[p1.x][p1.y] = new Queen(PlayerColor.WHITE);
			}
			return true;	
		}
		//Checks black pawn capture
		else if(this.owner == PlayerColor.BLACK && b.getBoard()[p2.x][p2.y] != null && p2.y == p1.y - 1
				&& b.getBoard()[p2.x][p2.y].owner == PlayerColor.WHITE && (p2.x == p1.x + 1 ||
				p2.x == p1.x - 1)){	
			if(p2.y == 0){		//Promotes the pawn to a queen if it reaches the other side
				b.getBoard()[p1.x][p1.y] = new Queen(PlayerColor.BLACK);
			}
			return true;	
		}
		//Checks white pawn capture
		else if(this.owner == PlayerColor.WHITE && b.getBoard()[p2.x][p2.y] != null && p2.y == p1.y + 1
				&& b.getBoard()[p2.x][p2.y].owner == PlayerColor.BLACK && (p2.x == p1.x + 1 ||
				p2.x == p1.x - 1)){
			if(p2.y == 7){		//Promotes the pawn to a queen if it reaches the other side
				b.getBoard()[p1.x][p1.y] = new Queen(PlayerColor.WHITE);
			}
			return true;
		}
		else{
			return false;
		}
	}
	
	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#getImage()
	 */
	@Override
	public Image getImage() {
		Image i = null;
		
		if(owner == PlayerColor.WHITE){
		i = new ImageIcon("chess/img/whitePawn.png").getImage();
		}
		else {
		i = new ImageIcon("chess/img/blackPawn.png").getImage();
		}
		
		return i;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		if(owner == PlayerColor.WHITE)
			return "p";
		else {
			return "P";
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	public Object clone() throws CloneNotSupportedException {
		return new Pawn(owner);
	}
	
	

	

}
