package chess.board.pieces;

import java.awt.Image;
import java.awt.Point;
import chess.board.Board;
import chess.board.Move;
import chess.board.PlayerColor;


/**Piece Class
 * @author Garon
 *
 */
public abstract class Piece {

	public PlayerColor owner;
	
	/**Constructor for Piece class
	 * @param owner	Owner of the Piece
	 */
	public Piece(PlayerColor owner) {
		this.owner = owner;		
	}
	
	
	/**Method that gets all the possible moves of a piece
	 * @param p1	Piece whose moves are asked for
	 * @param b		Board the piece is on
	 * @return		An array of moves
	 */
	public abstract Move[] getPossibleMoves(Point p1, Board b);

	
	/**Method that check whether a given move is valid
	 * 
	 * @param p1	Starting Point
	 * @param p2	Ending Point
	 * @param b		Board the piece is on
	 * @return		Returns true is piece can be moved, false if it can't
	 */
	public abstract boolean isMoveValid(Point p1, Point p2,  Board b);
	
	/**Gets the piece's image
	 * @return	Returns image of the piece
	 */
	public abstract Image getImage();
	
	
	/**Gets the piece's owner
	 * @return	Piece's Owner
	 */
	public PlayerColor getOwner() {
		return owner;
	}
	/**Sets the piece's owner
	 * @param owner	Piece's owner
	 */
	public void setOwner(PlayerColor owner) {
		this.owner = owner;
	}

	public abstract Object clone() throws CloneNotSupportedException;
}
