package chess.board.pieces;

import java.awt.Image;
import java.awt.Point;
import java.util.ArrayList;

import javax.swing.ImageIcon;

import chess.board.Board;
import chess.board.Move;
import chess.board.PlayerColor;

/**Queen Class, extends Piece
 * @author Garon
 *
 */
public class Queen extends Piece {

	/**Constructor for Queen
	 * @param owner	Owner of the Piece
	 */
	public Queen(PlayerColor owner) {
		super(owner);
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#getPossibleMoves(chess.board.Board)
	 */
	@Override
	public Move[] getPossibleMoves(Point p1, Board b) {
		ArrayList<Move> possibleMoves = new ArrayList<Move>();
		//check moves horizontal and vertical moves
		int xCounter = p1.x-1;
		while(xCounter >= 0 && b.getBoard()[xCounter][p1.y] == null){
			possibleMoves.add(new Move(p1, new Point(xCounter, p1.y)));
			xCounter--;
		}
		if(xCounter >= 0 && b.getBoard()[xCounter][p1.y].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(xCounter, p1.y)));
		}
		xCounter = p1.x+1;
		while(xCounter <= 7 && b.getBoard()[xCounter][p1.y] == null){
			possibleMoves.add(new Move(p1, new Point(xCounter, p1.y)));
			xCounter++;
		}
		if(xCounter <= 7 && b.getBoard()[xCounter][p1.y].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(xCounter, p1.y)));
		}
		int yCounter = p1.y-1;
		while(yCounter >= 0 && b.getBoard()[p1.x][yCounter] == null){
			possibleMoves.add(new Move(p1, new Point(p1.x, yCounter)));
			yCounter--;
		}
		if(yCounter >= 0 && b.getBoard()[p1.x][yCounter].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(p1.x, yCounter)));
		}
		yCounter = p1.y+1;
		while(yCounter <= 7 && b.getBoard()[p1.x][yCounter] == null){
			possibleMoves.add(new Move(p1, new Point(p1.x, yCounter)));
			yCounter++;
		}
		if(yCounter <= 7 && b.getBoard()[p1.x][yCounter].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(p1.x, yCounter)));
		}
		
		//check moves diagonal moves
		int diagCounter = 1;
		while(p1.x + diagCounter != 8 && p1.y + diagCounter != 8 &&
			  b.getBoard()[p1.x + diagCounter][p1.y + diagCounter] == null){
			possibleMoves.add(new Move(p1, new Point(p1.x + diagCounter, p1.y + diagCounter)));
			diagCounter++;
		}
		if(p1.x + diagCounter != 8 && p1.y + diagCounter != 8 &&
			  b.getBoard()[p1.x + diagCounter][p1.y + diagCounter].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(p1.x + diagCounter, p1.y + diagCounter)));
		}
		diagCounter = 1;
		while(p1.x - diagCounter != -1 && p1.y - diagCounter != -1 && 
			  b.getBoard()[p1.x - diagCounter][p1.y - diagCounter] == null){
			possibleMoves.add(new Move(p1, new Point(p1.x - diagCounter, p1.y - diagCounter)));
			diagCounter++;
		}
		if(p1.x - diagCounter != -1 && p1.y - diagCounter != -1 &&
		   b.getBoard()[p1.x - diagCounter][p1.y - diagCounter].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(p1.x - diagCounter, p1.y - diagCounter)));
		}
		diagCounter = 1;
		while(p1.x + diagCounter != 8 && p1.y - diagCounter != -1 && 
			  b.getBoard()[p1.x + diagCounter][p1.y - diagCounter] == null){
			possibleMoves.add(new Move(p1, new Point(p1.x + diagCounter, p1.y - diagCounter)));
			diagCounter++;
			}
		if(p1.x + diagCounter != 8 && p1.y - diagCounter != -1 &&
		   b.getBoard()[p1.x + diagCounter][p1.y - diagCounter].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(p1.x + diagCounter, p1.y - diagCounter)));
		}
		diagCounter = 1;
		while(p1.x - diagCounter != -1 && p1.y + diagCounter != 8 && 
			  b.getBoard()[p1.x - diagCounter][p1.y + diagCounter] == null){
			possibleMoves.add(new Move(p1, new Point(p1.x - diagCounter, p1.y + diagCounter)));
			diagCounter++;
			}
		if(p1.x - diagCounter != -1 && p1.y + diagCounter != 8 &&
		   b.getBoard()[p1.x - diagCounter][p1.y + diagCounter].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(p1.x - diagCounter, p1.y + diagCounter)));
		}		
		return (Move[]) possibleMoves.toArray(new Move[0]);
	}
	
	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#isMoveValid(java.awt.Point, java.awt.Point, chess.board.pieces.Piece[][])
	 */
	@Override
	public boolean isMoveValid(Point p1, Point p2, Board b) {
		boolean pieceInWay = false;			//used to check if there is a piece in the Bishop's path
		//Checks if the place the piece is moving is unocupied or occupied by an opponents piece
		//Queen's movement check is a combination of the Rook's and Bishop's
		//Check Rook's for comments on first two "if"s and Bishop's on second two
		if(p1.x == p2.x && ((b.getBoard()[p2.x][p2.y] == null) || (b.getBoard()[p2.x][p2.y].owner != this.owner))){
			int yMin;					//Same as Rook
			int yMax;
			if(p1.y < p2.y){
				yMin = p1.y+1;
				yMax = p2.y;
			}
			else{
				yMin = p2.y+1;
				yMax = p1.y;
			}
			while(yMin != yMax){
				if(b.getBoard()[p1.x][yMin] != null){
					pieceInWay = true;
				}
				yMin++;
			}			
			return !pieceInWay;			//returns true if the path is clear
		}
		else if(p1.y == p2.y && ((b.getBoard()[p2.x][p2.y] == null) || (b.getBoard()[p2.x][p2.y].owner != this.owner))){
			int xMin;					//Same as rook
			int xMax;
			if(p1.x < p2.x){
				xMin = p1.x+1;
				xMax = p2.x;
			}
			else{
				xMin = p2.x+1;
				xMax = p1.x;
			}
			while(xMin != xMax){
				if(b.getBoard()[xMin][p1.y] != null){
					pieceInWay = true;
				}
				xMin++;
			}			
			return !pieceInWay;			//returns true if the path is clear
		}
		else if( ((b.getBoard()[p2.x][p2.y] == null) || (b.getBoard()[p2.x][p2.y].owner != this.owner)) &&
			p1.x - p2.x == p1.y - p2.y){
			int moveDistance = p1.x - p2.x;		//Same as Bishop	
			if(moveDistance < 0){				
				int moveCounter = -1;
				while(moveCounter != moveDistance){
					if(b.getBoard()[p1.x-moveCounter][p1.y-moveCounter] != null){
						pieceInWay = true;						
					}
					moveCounter--;
				}
			}
			else{
				int moveCounter = 1;
				while(moveCounter != moveDistance){
					if(b.getBoard()[p1.x-moveCounter][p1.y-moveCounter] != null){
						pieceInWay = true;
					}
					moveCounter++;
				}				
			}			
			return !pieceInWay;			//returns true if the path is clear
		}
		else if(((b.getBoard()[p2.x][p2.y] == null) || (b.getBoard()[p2.x][p2.y].owner != this.owner)) &&
			p1.x - p2.x == -(p1.y - p2.y)){		//Same as Bishop
			int xMoveDistance = p1.x - p2.x;			
			if(xMoveDistance < 0){
				int moveCounter = -1;
				while(moveCounter != xMoveDistance){
					if(b.getBoard()[p1.x-moveCounter][p1.y+moveCounter] != null){
						pieceInWay = true;
					}
					moveCounter--;
				}
			}
			else{
				int moveCounter = 1;				
				while(moveCounter != xMoveDistance){
					if(b.getBoard()[p1.x-moveCounter][p1.y+moveCounter] != null){
						pieceInWay = true;
					}					
					moveCounter++;
				}				
			}
			return !pieceInWay;			//returns true if the path is clear
		}
		else {
			return false;
		}		
	}
	
	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#getImage()
	 */
	@Override
	public Image getImage() {
		Image i = null;
		
		if(owner == PlayerColor.WHITE){
		i = new ImageIcon("chess/img/whiteQueen.png").getImage();
		}
		else {
		i = new ImageIcon("chess/img/blackQueen.png").getImage();
		}
		
		return i;
		
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		if(owner == PlayerColor.WHITE)
			return "q";
		else {
			return "Q";
		}
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		return new Queen(owner);
	}



}
