package chess.board.pieces;

import java.awt.Image;
import java.awt.Point;
import java.util.ArrayList;

import javax.swing.ImageIcon;

import chess.board.Board;
import chess.board.Move;
import chess.board.PlayerColor;

/**Rook Class, extends Piece
 * @author Garon
 *
 */
public class Rook extends Piece {

	/**Constructor for Rook
	 * @param owner	Owner of the Piece
	 */
	public Rook(PlayerColor owner) {
		super(owner);
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#getPossibleMoves(chess.board.Board)
	 */
	@Override
	public Move[] getPossibleMoves(Point p1, Board b) {
		ArrayList<Move> possibleMoves = new ArrayList<Move>();
		int xCounter = p1.x-1;
		while(xCounter >= 0 && b.getBoard()[xCounter][p1.y] == null){
			possibleMoves.add(new Move(p1, new Point(xCounter, p1.y)));
			xCounter--;
		}
		if(xCounter >= 0 && b.getBoard()[xCounter][p1.y].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(xCounter, p1.y)));
		}
		xCounter = p1.x+1;
		while(xCounter <= 7 && b.getBoard()[xCounter][p1.y] == null){
			possibleMoves.add(new Move(p1, new Point(xCounter, p1.y)));
			xCounter++;
		}
		if(xCounter <= 7 && b.getBoard()[xCounter][p1.y].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(xCounter, p1.y)));
		}
		int yCounter = p1.y-1;
		while(yCounter >= 0 && b.getBoard()[p1.x][yCounter] == null){
			possibleMoves.add(new Move(p1, new Point(p1.x, yCounter)));
			yCounter--;
		}
		if(yCounter >= 0 && b.getBoard()[p1.x][yCounter].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(p1.x, yCounter)));
		}
		yCounter = p1.y+1;
		while(yCounter <= 7 && b.getBoard()[p1.x][yCounter] == null){
			possibleMoves.add(new Move(p1, new Point(p1.x, yCounter)));
			yCounter++;
		}
		if(yCounter <= 7 && b.getBoard()[p1.x][yCounter].owner != this.owner){
			possibleMoves.add(new Move(p1, new Point(p1.x, yCounter)));
		}		
		return (Move[]) possibleMoves.toArray(new Move[0]);
	}
	
	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#isMoveValid(java.awt.Point, java.awt.Point, chess.board.pieces.Piece[][])
	 */
	@Override
	public boolean isMoveValid(Point p1, Point p2, Board b) {
		
		boolean pieceInWay = false;		//used to check if there is a piece in the Rook's path
		//Checks if the place the piece is moving is unocupied or occupied by an opponents piece
		if(p1.x == p2.x && ((b.getBoard()[p2.x][p2.y] == null) || (b.getBoard()[p2.x][p2.y].owner != this.owner))){
			int yMin;
			int yMax;
			if(p1.y < p2.y){			//If piece is moving along the vertical
				yMin = p1.y+1;			//Checks the vertical for pieces in the way
				yMax = p2.y;
			}
			else{
				yMin = p2.y+1;
				yMax = p1.y;
			}
			while(yMin != yMax){
				if(b.getBoard()[p1.x][yMin] != null){
					pieceInWay = true;
				}
				yMin++;
			}
			return !pieceInWay;			//returns true if the path is clear
		}
		else if(p1.y == p2.y && ((b.getBoard()[p2.x][p2.y] == null) || (b.getBoard()[p2.x][p2.y].owner != this.owner))){
			int xMin;
			int xMax;
			if(p1.x < p2.x){			//If piece is moving along horizontal
				xMin = p1.x+1;			//Checks the horizontal for pieces in the way
				xMax = p2.x;
			}
			else{
				xMin = p2.x+1;
				xMax = p1.x;
			}
			while(xMin != xMax){
				if(b.getBoard()[xMin][p1.y] != null){
					pieceInWay = true;
				}
				xMin++;
			}
			return !pieceInWay;			//returns true if the path is clear
		}
		else{
			return false;
		}
	}
	
	/* (non-Javadoc)
	 * @see chess.board.pieces.Piece#getImage()
	 */
	@Override
	public Image getImage() {
		Image i = null;
		
		if(owner == PlayerColor.WHITE){
		i = new ImageIcon("chess/img/whiteRook.png").getImage();
		}
		else {
		i = new ImageIcon("chess/img/blackRook.png").getImage();
		}
		
		return i;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		if(owner == PlayerColor.WHITE)
			return "r";
		else {
			return "R";
		}
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		return new Rook(owner);
	}



}
