package chess.client;

import java.awt.Point;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.StringTokenizer;

import chess.board.Board;
import chess.board.Move;
import chess.board.PlayerColor;
import chess.graphics.GuiBoard;
import chess.graphics.GuiWinBox;

/**Gui Chess client to be used on the server
 * @author Garon
 * 
 */
public class GuiChessClient {
	private static Socket socket = null;

	private static PrintWriter out = null;

	private static BufferedReader in = null;

	private static PlayerColor owner;

	private static GuiBoard guiBoard; // This player's copy of the board

	/**Constructor for the Client
	 * 
	 */
	public GuiChessClient() {
		super();
	}
	
	/**Starts the Client listening to the socket
	 * @param ip		Ip address to locate server at
	 * @param port		port to connect to
	 */
	public void listenSocket(String ip, int port) {
		// Create socket connection
		try {
			socket = new Socket(ip, port);
			out = new PrintWriter(socket.getOutputStream(), true);
			in = new BufferedReader(new InputStreamReader(socket
					.getInputStream()));
		} catch (UnknownHostException e) {
			System.out.println("Unknown host: ganges");
			System.exit(1);
		} catch (IOException e) {
			System.out.println("No I/O");
			System.exit(1);
		}
	}

	/**Parses a string into a readable move
	 * @param line		Line read in to be parsed as a move
	 * @return	Move	parsed move
	 */
	public Move parseMove(String line) {
		StringTokenizer str = new StringTokenizer(line);
		String[] points = new String[4];
		int counter = 0;
		while (str.hasMoreTokens()) {
			points[counter] = str.nextToken();
			counter++;
		}
		Point p1 = new Point(Integer.parseInt(points[0]), Integer
				.parseInt(points[1]));
		Point p2 = new Point(Integer.parseInt(points[2]), Integer
				.parseInt(points[3]));
		return new Move(p1, p2);
	}

	/**Sends a move to the server
	 * @param m	move to be sent
	 */
	public void sendMove(Move m) {
		String move = m.startPoint.x + " " + m.startPoint.y + ' '
				+ m.endPoint.x + ' ' + m.endPoint.y;
		out.println(move);
	}

	/**Gets a move from the action on a Gui board
	 * @return		move made on the board
	 */
	public static Move getMoveFromBoard() {
		guiBoard.yourTurn = true;
		guiBoard.inCheck = guiBoard.getMyBoard().isCheck(owner);
		guiBoard.repaint();
		while (!guiBoard.moveSet) {
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		Move move = guiBoard.getCurrentMove();
		guiBoard.getMyBoard().movePiece(owner, move.startPoint, move.endPoint);
		guiBoard.repaint();
		guiBoard.moveSet = false;
		guiBoard.yourTurn = false;
		guiBoard.inCheck = false;
		return move;

	}

	/**Main chess Client
	 * @param args
	 */
	public static void main(String[] args) {
		GuiChessClient g = new GuiChessClient();
		
		//get IP and port
		GuiClientStartBox g1 = new GuiClientStartBox();
		g1.setVisible(true);
		
		while(!g1.isSet){
			try {
				Thread.sleep(50);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}			
		}
		
		
		g.listenSocket(g1.ipAddress, g1.portNumber);

		try {
			String playerColor = in.readLine();
			if (playerColor.equals("BLACK")) {
				owner = PlayerColor.BLACK;
			} else if (playerColor.equals("WHITE")) {
				owner = PlayerColor.WHITE;
			}
		} catch (IOException e1) {
			e1.printStackTrace();
		}

		guiBoard = new GuiBoard(owner + " Chess Board");
		guiBoard.setVisible(true);

		boolean gameOver = false;

		while (!gameOver) {
			String line = null;
			try {
				line = in.readLine();

				if (line.startsWith("Game won:")) {
					int startIndex = line.indexOf(':') + 1;
					if (line.substring(startIndex).equals("WHITE")) {
						String winner = "WHITE";
						GuiWinBox g2 = new GuiWinBox(winner);
						g2.setVisible(true);
					} else {
						String winner = "BLACK";
						GuiWinBox g2 = new GuiWinBox(winner);
						g2.setVisible(true);
					}
					gameOver = true;
				} else if (line.equals("Request Move")) {
					Move startMove;
					startMove = getMoveFromBoard();
					g.sendMove(startMove);
				} else {
					Move m = g.parseMove(line);
					guiBoard.getMyBoard().movePiece(Board.switchColor(owner),
							m.startPoint, m.endPoint);
				}
			} catch (IOException e) {
				System.out.println(e);
			}
		}
	}

}
