package chess.graphics;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;

import chess.player.PlayDifficulty;
import chess.player.PlayStyle;

/**Start Screen for Chess Game
 * @author Garon
 *
 */
public class GuiStartScreen extends JFrame {
	
	private static final long serialVersionUID = -7129429067272226030L;
	
	public boolean isSet = false;
	public PlayDifficulty whiteDiff;
	public PlayDifficulty blackDiff;
	public PlayStyle whiteStyle;
	public PlayStyle blackStyle;
	
	public PlayerSettingPanel whitePanel;
	public PlayerSettingPanel blackPanel;

	/**Constructor for Start Screen
	 * @throws HeadlessException
	 */
	public GuiStartScreen() throws HeadlessException {
		super();
		createUI();
		repaint();
	}	

	/**Constructor for Start Screen
	 * @param arg0	Title of Screen
	 * @throws HeadlessException
	 */
	public GuiStartScreen(String arg0) throws HeadlessException {
		super(arg0);
		createUI();
		repaint();
	}
	
	/**Creates the UI for the start screen
	 * 
	 */
	protected void createUI() {

		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				dispose();
			}
		});
		
		//Create and set up the content pane.
        JComponent newContentPane = new JPanel();
        newContentPane.setLayout(new BoxLayout(newContentPane, BoxLayout.Y_AXIS));
        newContentPane.setOpaque(true); //content panes must be opaque
        this.setContentPane(newContentPane);
		
		whitePanel = new PlayerSettingPanel(new GridLayout(1, 0));
		whitePanel.setBorder(BorderFactory.createTitledBorder("White Player"));
		newContentPane.add(whitePanel,BorderLayout.CENTER);
		
		blackPanel = new PlayerSettingPanel(new GridLayout(1, 0));
		blackPanel.setBorder(BorderFactory.createTitledBorder("Black Player"));
		newContentPane.add(blackPanel,BorderLayout.CENTER);		
		
		JButton okButton = new JButton("Ok");					
		
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String whiteDiffCheck = (String) whitePanel.aiDifficulty.getSelectedItem();
				if(whiteDiffCheck == "Easy"){
					whiteDiff = PlayDifficulty.EASY;
				}
				else if(whiteDiffCheck == "Medium"){
					whiteDiff = PlayDifficulty.MEDIUM;
				}
				else if(whiteDiffCheck == "Hard"){
					whiteDiff = PlayDifficulty.HARD;
				}
				String whiteStyleCheck = (String) whitePanel.aiStyle.getSelectedItem();
				if(whiteStyleCheck == "Neutral"){
					whiteStyle = PlayStyle.NEUTRAL;
				}
				else if(whiteStyleCheck == "Defensive"){
					whiteStyle = PlayStyle.DEFENSIVE;
				}
				else if(whiteStyleCheck == "Aggressive"){
					whiteStyle = PlayStyle.AGGRESSIVE;
				}
				String blackDiffCheck = (String) blackPanel.aiDifficulty.getSelectedItem();
				if(blackDiffCheck == "Easy"){
					blackDiff = PlayDifficulty.EASY;
				}
				else if(blackDiffCheck == "Medium"){
					blackDiff = PlayDifficulty.MEDIUM;
				}
				else if(blackDiffCheck == "Hard"){
					blackDiff = PlayDifficulty.HARD;
				}
				String blackStyleCheck = (String) blackPanel.aiStyle.getSelectedItem();
				if(blackStyleCheck == "Neutral"){
					blackStyle = PlayStyle.NEUTRAL;
				}
				else if(blackStyleCheck == "Defensive"){
					blackStyle = PlayStyle.DEFENSIVE;
				}
				else if(blackStyleCheck == "Aggressive"){
					blackStyle = PlayStyle.AGGRESSIVE;
				}				
				isSet = true;				
				dispose();			    
		    }			
		});		
		newContentPane.add(okButton, BorderLayout.CENTER);

        //Display the window.
        this.pack();
		center();
	}
	
	/**Centers the Gui on the screen
	 * 
	 */
	public void center() {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = getSize();
		int x = (screenSize.width - frameSize.width) / 2;
		int y = (screenSize.height - frameSize.height) / 2;
		setLocation(x, y);
	}


}
