package chess.graphics;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.HeadlessException;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

public class GuiWinBox extends JFrame {

	private static final long serialVersionUID = 558150557667596035L;

	/**Box displayed upon checkmate
	 * @throws HeadlessException
	 */
	public GuiWinBox() throws HeadlessException {
		super();
		createUI("");
	}
	
	/**Box displayed upon checkmate
	 * @param arg0		winner of the game
	 * @throws HeadlessException
	 */
	public GuiWinBox(String arg0) throws HeadlessException {
		super(arg0);
		createUI(arg0);
	}

	/**Creates the UI for the box
	 * 
	 */
	protected void createUI(String s) {

		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				dispose();
			}
		});
		
		//Create and set up the content pane.
        JComponent newContentPane = new JPanel();
        newContentPane.setLayout(new BoxLayout(newContentPane, BoxLayout.Y_AXIS));
        newContentPane.setOpaque(true); //content panes must be opaque
        this.setContentPane(newContentPane);
		
		String winner = "Checkmate! " + s + " Wins!";
		JLabel winMessage = new JLabel(winner, JLabel.CENTER);
		winMessage.setVerticalTextPosition(JLabel.CENTER);
		winMessage.setHorizontalTextPosition(JLabel.CENTER);
		Font f = new Font("Arial", Font.BOLD, 36);
		winMessage.setFont(f);
		newContentPane.add(winMessage,BorderLayout.CENTER);

        //Display the window. 
		this.pack();
		center();
	}
	
	/**Centers the Gui on the screen
	 * 
	 */
	public void center() {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = getSize();
		int x = (screenSize.width - frameSize.width) / 2;
		int y = (screenSize.height - frameSize.height) / 2;
		setLocation(x, y);
	}


}
