package chess.player;

import chess.board.Board;
import chess.board.Move;
import chess.board.PlayerColor;
import chess.graphics.GuiBoard;
import chess.graphics.GuiWinBox;

/**GUI Player, extends player class
 * @author Garon
 *
 */
public class GuiPlayer extends Player {
	private GuiBoard guiBoard;	//This player's copy of the board
	

	/**Constructor for the GUI Player
	 * @param owner		this players color
	 */
	public GuiPlayer(PlayerColor owner) {
		super(owner);
		guiBoard = new GuiBoard(owner + " Chess Board");
	}

	/* (non-Javadoc)
	 * @see chess.player.Player#getMove()
	 */
	@Override
	public Move getMove() {
		guiBoard.yourTurn = true;
		guiBoard.inCheck = guiBoard.getMyBoard().isCheck(this.owner);
		guiBoard.repaint();
		while(!guiBoard.moveSet){
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		Move move = guiBoard.getCurrentMove();
		guiBoard.getMyBoard().movePiece(getOwner(), move.startPoint, move.endPoint);
		guiBoard.repaint();
		guiBoard.moveSet = false;
		guiBoard.yourTurn = false;
		guiBoard.inCheck = false;
		return move;
	}

	/* (non-Javadoc)
	 * @see chess.player.Player#sendMove(chess.board.Move)
	 */
	@Override
	public void sendMove(Move m) {
		guiBoard.getMyBoard().movePiece(Board.switchColor(getOwner()),m.startPoint, m.endPoint);
	}

	/* (non-Javadoc)
	 * @see chess.player.Player#startGame()
	 */
	@Override
	public void startGame(PlayerColor owner) {
		guiBoard.setVisible(true);	//Makes the Board playable for a GUI player	
	}

	@Override
	public void gameTerminated(boolean isWinner) {
		if(isWinner){
			String winner = "" + this.owner;
			GuiWinBox g2 = new GuiWinBox(winner);
			g2.setVisible(true);
		}
		else if(!isWinner){
			String winner = "" + Board.switchColor(this.owner);
			GuiWinBox g2 = new GuiWinBox(winner);
			g2.setVisible(true);
		}
		
	}

}
