package chess.player;

import java.awt.Point;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.StringTokenizer;

import chess.board.Board;
import chess.board.Move;
import chess.board.PlayerColor;

/**Network Player class, extends Player
 * @author Garon
 *
 */
public class NetworkPlayer extends Player {
	ServerSocket server = null;
	Socket client = null;
	BufferedReader in = null;
	PrintWriter out = null;

	/**Constructor for the Player
	 * @param owner		Color of the player
	 */
	public NetworkPlayer(PlayerColor owner) {
		super(owner);		
	}

	/* (non-Javadoc)
	 * @see chess.player.Player#getMove()
	 */
	@Override
	public Move getMove() {
		try {
			out.println("Request Move");
			
			String line = in.readLine();
			// Send data back to client
			System.out.println(line);
			//out.println(line);
			StringTokenizer str = new StringTokenizer(line);
			String[] points = new String[4];
			int counter = 0;
			while(str.hasMoreTokens()){
				points[counter] = str.nextToken();
				counter++;
			}
			Point p1 = new Point(Integer.parseInt(points[0]), Integer.parseInt(points[1]));
			Point p2 = new Point(Integer.parseInt(points[2]), Integer.parseInt(points[3]));			
			return new Move(p1,p2 );			
		} catch (IOException e) {
			System.out.println("Read failed");		
		}				
		return null;
	}

	/* (non-Javadoc)
	 * @see chess.player.Player#sendMove(chess.board.Move)
	 */
	@Override
	public void sendMove(Move m) {
		String move = m.startPoint.x + " " + m.startPoint.y + ' ' + m.endPoint.x + ' ' + m.endPoint.y;			
		out.println(move);
	}

	/* (non-Javadoc)
	 * @see chess.player.Player#startGame()
	 */
	@Override
	public void startGame(PlayerColor owner) {
		try {
			if(owner == PlayerColor.WHITE){
				server = new ServerSocket(4444);
			}
			else{
				server = new ServerSocket(4445);
			}
		} catch (IOException e) {
			System.out.println("Could not listen on port 4444");
			System.exit(-1);
		}

		try {
			client = server.accept();
		} catch (IOException e) {
			System.out.println("Accept failed: 4444");
			System.exit(-1);
		}

		try {
			in = new BufferedReader(new InputStreamReader(client
					.getInputStream()));
			out = new PrintWriter(client.getOutputStream(), true);
		} catch (IOException e) {
			System.out.println("Accept failed: 4444");
			System.exit(-1);
		}
		
		out.println(owner);

//		while (true) {
//			try {
//				String line = in.readLine();
//				// Send data back to client
//				System.out.println(line);
//				out.println(line);
//			} catch (IOException e) {
//				System.out.println("Read failed");
//				System.exit(-1);
//			}
//		}

	}

	@Override
	public void gameTerminated(boolean isWinner) {
		if(isWinner){
			String winner = "Game won:" + this.owner;
			out.println(winner);
		}
		else if(!isWinner){
			String winner = "Game won:" + Board.switchColor(this.owner);
			out.println(winner);
		}
		try {
			if(client != null){
				client.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		try {
			if(server != null){
				server.close();
			}
		} catch (IOException e) {	
			e.printStackTrace();
		}
	}

}
