package chess.player;

import chess.board.Move;
import chess.board.PlayerColor;

/**Player Class
 * @author Garon
 *
 */
public abstract class Player {

	protected PlayerColor owner;	//Which player this is
	
	/**Constructor for the Player class
	 * @param owner	Which player this is
	 */
	public Player(PlayerColor owner){
		this.owner = owner;		
	}
	
	/**Gets the owner of the player
	 * @return	Either black or white
	 */
	public PlayerColor getOwner() {
		return owner;
	}

	/**Sets the owner of the player
	 * @param playerColor	Color to be set to the player
	 */
	public void setOwner(PlayerColor playerColor) {
		this.owner = playerColor;
	}
	
	/**Starts the game for this player
	 * 
	 */
	public abstract void startGame(PlayerColor owner);

	/**Gets a move from the player for the server
	 * @return	Move to be sent to the Server
	 */
	public abstract Move getMove();
	
	/**Sends a move to the player from the server
	 * @param m	Move sent to the player
	 */
	public abstract void sendMove(Move m);
	
	/**
	 * @param isWinner
	 */
	public abstract void gameTerminated(boolean isWinner);
}
