package chess.player.heuristics;

import chess.board.Board;
import chess.board.PlayerColor;
import chess.board.pieces.Bishop;
import chess.board.pieces.King;
import chess.board.pieces.Knight;
import chess.board.pieces.Pawn;
import chess.board.pieces.Piece;
import chess.board.pieces.Queen;
import chess.board.pieces.Rook;

/**Aggressive Heuristic for AI player
 * @author Garon
 *
 */
public class Aggressive implements Heuristic {

	/* (non-Javadoc)
	 * @see chess.player.heuristics.Heuristic#evaluateBoardState(chess.board.PlayerColor, chess.board.Board)
	 */
	public int evaluateBoardState(PlayerColor c, Board b) {
		int sum = 0;
		
		Piece[][] pieces = b.getBoard();
		for(int i = 0; i < 8; i++) {
			for(int j = 0; j < 8; j++) {
				Piece current = pieces[j][i];
				if(current != null) {
					int multiplier = 1;
					int value = 0;
					int playStyleMultiplier = 1;
					if(c != current.owner) {
						playStyleMultiplier = 3;
					}
					
					if(current.owner == PlayerColor.WHITE) {
						multiplier = 1;
					} else {
						multiplier = -1;
					}
					//Material Calculations
					if(current instanceof King) {
						value = 2000;
					} 
					else if(current instanceof Queen) {
						value = 50;
					} 
					else if(current instanceof Rook) {
						value = 25;
					} 
					else if(current instanceof Bishop) {
						value = 15;
					} 
					else if(current instanceof Knight) {
						value = 15;
					} 
					else if(current instanceof Pawn) {
						value = 1;
					}
					sum += (playStyleMultiplier * multiplier * value);
				}
			}
		}
		if(b.isCheck(Board.switchColor(c))){
			int value = 0;
			if(c == PlayerColor.WHITE){
				value = 90;
			}
			else if( c == PlayerColor.BLACK){
				value = -90;
			}
			sum += value;
		}
		return sum;
	}

}
